#!/usr/bin/python
#coding:latin1
import os

TAB=2*' '
PDEFAULT="Default"

MODELPATH = 'model/model.txt'
XMIPATH = "xmi/ClassModel001.emx"
STYLE = 1 #nested class statt package 

def makedirs(path):
    try:
        os.makedirs(path)
    except Exception, e:
        #print e
        pass

def gen_id():
    gen_id.n+=1
    return gen_id.n
gen_id.n = 99999

def header(name, id0, id1, id2):
    return """\
<?xml version="1.0" encoding="UTF-8"?>
<!--xtools2_universal_type_manager-->
<?com.ibm.xtools.emf.core.signature <signature id="com.ibm.xtools.uml.msl.model" version="7.0.0"><feature description="" name="com.ibm.xtools.ruml.feature" url="" version="7.0.0"/></signature>?>
<?com.ibm.xtools.emf.core.signature <signature id="com.ibm.xtools.mmi.ui.signatures.diagram" version="7.0.0"><feature description="" name="Rational Modeling Platform (com.ibm.xtools.rmp)" url="" version="7.0.0"/></signature>?>
<uml:Model xmi:version="2.0" xmlns:xmi="http://www.omg.org/XMI" xmlns:ecore="http://www.eclipse.org/emf/2002/Ecore" xmlns:uml="http://www.eclipse.org/uml2/2.0.0/UML" xmi:id="%(_id0)s" name="%(_name)s">
  <eAnnotations xmi:id="%(_id1)s" source="uml2.diagrams"/>
  <packageImport xmi:id="%(_id2)s">
    <importedPackage xmi:type="uml:Model" href="pathmap://UML_LIBRARIES/UMLPrimitiveTypes.library.uml#_0"/>
  </packageImport>""" % dict(_name=name, _id0=id0, _id1=id1, _id2=id2)      


def footer(id0, id1, id2, id3, id4, id5):
    return"""\
</uml:Model> """ % dict(_id0=id0, _id1=id1, _id2=id2, _id3=id3, _id4=id4, _id5=id5)      

xmidir, xminame = os.path.split(XMIPATH)

makedirs(xmidir)
registry={}
handled_classes={}

class Attr:
    def __init__(self, name, visibility, type):
        self.id=gen_id()
        self.name=name
        self.visibility=visibility
        self.type=type
    def show(self, fw, indent):        
        if fw: fw.write('%s<ownedAttribute  xmi:id="%s" name="%s" visibility="%s" type="%s"/>\n' % (indent*TAB, self.id, self.name, self.visibility, self.type.id))
        
class Reference:
    def __init__(self, name, xmi_type, href):
        self.name=name
        self.xmi_type=xmi_type
        self.href = href

STYLE_NESTED_CLASS = True
ATTRIBUTES = True
METHODS = True
class Package:
    xmi_type = "uml:Package" 
    def __init__(self, name):
        key2 = name
        self.id = registry.setdefault(self.xmi_type, {}).setdefault(key2, gen_id())        
        self.name = name
        self.classes=[]
    def show(self, fw, indent):
        xmi_type = STYLE_NESTED_CLASS and "uml:Class" or "uml:Package" 
        if fw: fw.write('%s<packagedElement xmi:type="%s" xmi:id="%s" name="%s">\n' % (indent*TAB,  xmi_type, self.id, self.name))
        for cls in self.classes:
            cls.show(fw, indent+1)
        if fw: fw.write('%s</packagedElement>\n' % (indent*TAB))

class Class:
    xmi_type="uml:Class"
    def __init__(self, name, pname):
        self.key2 = (name, pname)
        self.id = registry.setdefault(self.xmi_type, {}).setdefault(self.key2, gen_id())
        self.name=name
        self.pname=pname
        self.bases=[]
        self.methods=[]
        self.attrs=[]
    def show(self, fw, indent):
        tagname = STYLE_NESTED_CLASS and "nestedClassifier" or "packagedElement"
        if fw: fw.write('%s<%s xmi:type="%s" xmi:id="%s" name="%s">\n' % (indent*TAB, tagname, self. xmi_type, self.id, self.name))
        for base in self.bases:
            bkey2 = base.split('.')
            bkey2.reverse()
            if len(bkey2) == 1:
                bkey2.append(PDEFAULT)
            assert len(bkey2)==2, bkey2
            bkey2 = tuple(bkey2)            
            base_id = registry.setdefault(self.xmi_type, {}).setdefault(bkey2, gen_id())            
            #print 999, self.xmi_type, bkey2, registry[self.xmi_type][bkey2]
            
            if fw: fw.write('%s<generalization xmi:id="%s" general="%s"/>\n' % ((1+indent)*TAB, gen_id(),base_id))
        for a in self.attrs:
            a.show(fw, indent+1)
        for m in self.methods:
            m.show(fw, indent+1)            
        if fw: fw.write('%s</%s>\n' % (indent*TAB, tagname))
        handled_classes[self.key2]=True
                        
class DataType:
    xmi_type="uml:DataType"
    def __init__(self, name):        
        key2 = name
        self.id = registry.setdefault(self.xmi_type, {}).setdefault(key2, gen_id())     
        self.name=name

class Param:
    def __init__(self, name, type):
        self.id=gen_id()
        self.name=name
        self.type=type
    def show(self, fw, indent):
        if fw: fw.write('%s<ownedParameter  xmi:id="%s" name="%s"/>\n' % (indent*TAB, self.id, self.name))        
        
class Method:
    def __init__(self, name, visibility):
        self.id=gen_id()
        self.name=name
        self.return_type = None
        self.params=[]
        self.visibility=visibility
    def show(self, fw, indent):
        if fw: fw.write('%s<ownedOperation  xmi:id="%s" name="%s">\n' % (indent*TAB, self.id, self.name))
        for param in self.params:
            param.show(fw, indent+1)
        if fw: fw.write('%s</ownedOperation>\n' % (indent*TAB))    

               
def transform():
    fr = open(MODELPATH)
    packages=[]
    
    for line in fr:        
        tp, value = line[0], line[1:].strip()
        
        if tp == 'P':    
            p = Package(name=value)
            packages.append(p)
        elif tp == 'C':                        
            c = Class(name=value, pname=p.name)
            p.classes.append(c)
        elif tp == 'B':
            c.bases.append(value)
        elif tp == 'a':
            if ATTRIBUTES: c.attrs.append(Attr(name=value, visibility="private", type=DataType('?')))             
        elif tp == 'm':
            if METHODS:
                m = Method(name=value, visibility="public")            
                c.methods.append(m)
        elif tp == 'p':
            if METHODS:
                m.params.append(Param(value, type=DataType('?')))
        
    fr.close() 
    return packages

def transform2(packages):
    classes = registry["uml:Class"]
    pkg_default=None
    for key2 in classes:        
        if key2 not in handled_classes:
            #print key2, 
            assert len(key2) == 2, key2
            if not pkg_default:
                pkg_default = Package(name=PDEFAULT)
                packages.insert(0, pkg_default)
            cname, pname=key2
            print 777, key2                       
            c = Class(name=cname, pname=pname)
            pkg=None
            for p in packages:
                if p.name == pname:
                    pkg=p
                    break
            assert(pkg)

            print 888, key2, classes[key2], c.name, c.pname 
            pkg.classes.append(c)
    return packages
    
def main():            
    fw = open(XMIPATH,'w')    
    if fw: fw.write(header("XBRL-Converter", gen_id(), gen_id(), gen_id()))
    packages =transform()    
    indent=1
    for package in packages:
        package.show(None, indent)  
    packages = transform2(packages)
    for package in packages:
        package.show(fw, indent)
        
    xmi_type = "uml:DataType"
    for (key2, xmi_id) in registry.get(xmi_type,{}).items():
        name = key2
        if fw: fw.write('%s<packagedElement xmi:type="%s" xmi:id="%s" name="%s"/>\n' % (indent*TAB, xmi_type, xmi_id, name))
        #print '%s<packagedElement xmi:type="%s" xmi:id="%s" name="%s"/>\n' % (indent*TAB, xmi_type, xmi_id, name)


    if fw: fw.write(footer(gen_id(), gen_id(), gen_id(), gen_id(), gen_id(), gen_id()))
    fw. close()
main()      

